(* $Id: rpc.mli 1003 2006-09-24 15:17:15Z gerd $
 * ----------------------------------------------------------------------
 *
 *)

(** Common types and exceptions *)

open Rtypes

type protocol =
    Tcp          (** means: stream-oriented connection *)
  | Udp;;        (** means: datagram exchange *)

type mode =
    Socket     (** classical server socket *)
  | BiPipe     (** server is endpoint of a bidirectional pipe *)

(* these are error conditions sent to the client: *)

type server_error =
    Unavailable_program                      (** accepted call! *)
  | Unavailable_version of (uint4 * uint4)   (** accepted call  *)
  | Unavailable_procedure                    (** accepted call  *)
  | Garbage                                  (** accepted call  *)
  | System_err                               (** accepted call  *)
  | Rpc_mismatch of (uint4 * uint4)          (** rejected call  *)
  | Auth_bad_cred                            (** rejected call  *)
  | Auth_rejected_cred                       (** rejected call  *)
  | Auth_bad_verf                            (** rejected call  *)
  | Auth_rejected_verf                       (** rejected call  *)
  | Auth_too_weak                            (** rejected call  *)
  | Auth_invalid_resp                        (** rejected call  *)
  | Auth_failed                              (** rejected call  *)
;;

exception Rpc_server of server_error;;
  (** an exception generated by the RPC server *)


exception Rpc_cannot_unpack of string;;
  (** RPC protocol error (bad data) *)

